﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace TreeExample
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void quitMenu_Click(object sender, EventArgs e)
        {
            Close ();
        }

        private void addDir (TreeNode top, DirectoryInfo dir, int level)
        {
            TreeNode node = new TreeNode();
            node.Text = dir.Name;
            node.ToolTipText = dir.FullName;

            if (top == null)
                tree.Nodes.Add (node);
            else
                top.Nodes.Add (node);

            if (level > 0)
               try
               {
               foreach (DirectoryInfo d in dir.GetDirectories())
                  addDir (node, d, level-1);
               }
               catch (Exception ex)
               {
               }
        }

        private void testMenu_Click(object sender, EventArgs e)
        {
            // using System.IO;
            DirectoryInfo dir = new DirectoryInfo("C:\\");
            addDir (null, dir, 3);
            /*
            TreeNode root = new TreeNode ();
            root.Text = "root";
            tree.Nodes.Add (root);

            TreeNode node = new TreeNode();
            node.Text = "subitem";
            root.Nodes.Add (node);
            */

        }

        TreeNode source = null;

        private void dragAndDropButton_MouseDown(object sender, MouseEventArgs e)
        {
            source = null;
            DoDragDrop("nejaka data", DragDropEffects.Copy |
                                      DragDropEffects.Move |
                                      DragDropEffects.Link);
        }

        private void tree_MouseDown(object sender, MouseEventArgs e)
        {
            source = tree.SelectedNode;
            if (source != null && e.Button == MouseButtons.Right)
                DoDragDrop("jina data", DragDropEffects.Copy |
                                          DragDropEffects.Move |
                                          DragDropEffects.Link);
        }

        private void tree_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            tree.SelectedNode = e.Node;
        }

        private void tree_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }

        private void tree_DragOver(object sender, DragEventArgs e)
        {
            Point pt = tree.PointToClient(new Point (e.X, e.Y));
            TreeNode node = tree.GetNodeAt (pt);
            tree.SelectedNode = node;

            if (node != null && 
                Control.ModifierKeys == Keys.Control &&
                node.Nodes.Count != 0 && 
                ! node.IsExpanded)
                   node.Expand ();

            if (Control.ModifierKeys == Keys.Shift)
                e.Effect = DragDropEffects.Copy;
            else if (Control.ModifierKeys == Keys.Alt)
                e.Effect = DragDropEffects.Link;
            else
                e.Effect = DragDropEffects.Move;


            if (source != null && e.Effect == DragDropEffects.Move)
            {
               bool ok = true;
               TreeNode p = tree.SelectedNode;
               while (p != null && ok)
               {
                   if (p == source) ok = false;
                   p = p.Parent;
               }
               if (!ok)
                   e.Effect = DragDropEffects.None;
            }
        }

        private void tree_DragDrop(object sender, DragEventArgs e)
        {
            TreeNode target = tree.SelectedNode;
            
            if (source == null)
            {
               TreeNode node = new TreeNode();
               node.Text = "new item";
            
               if (target == null)
                  tree.Nodes.Add(node);
               else
               {
                  target.Nodes.Insert (0, node);
                  target.Expand ();
               }
            }
            else if (e.Effect == DragDropEffects.Move)
            {
                bool ok = true;
                TreeNode p = target;
                while (p != null && ok)
                {
                    if (p == source) ok = false;
                    p = p.Parent;
                }

                if (ok)
                {
                   source.Remove();

                   if (target == null)
                       tree.Nodes.Add(source);
                   else
                   {
                       target.Nodes.Insert(0, source);
                       target.Expand();
                   }
                }
            }
            else if (e.Effect == DragDropEffects.Copy)
            {
                TreeNode node = TreeCopy (source);

                if (target == null)
                    tree.Nodes.Add (node);
                else
                {
                    target.Nodes.Insert (0, node);
                    target.Expand();
                }
            }
        }

        private TreeNode TreeCopy (TreeNode from)
        {
            TreeNode result = new TreeNode ();
            result.Text = from.Text;

            foreach (TreeNode item in from.Nodes)
            {
                TreeNode copy = TreeCopy (item);
                result.Nodes.Add (copy);
            }

            return result;
        }


    }
}
